////////////////////////////////////////////////////////////////////////////
//
//  uParse.c
//
//	Implementierung des Parsers
//
//	Author 	: Rene Bllhoff
//  Created : around Okt. 2007
//
//	(c) 2005-2008 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
//
//////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
////////////////////////////////////////////////////////////////////////////	
	
	#include "lib.h"


	// makros um anzahl der elemente eines arrays zu ermitteln

	#define NUM_ARRAY_ELEMENTS(x)   ( sizeof ( (x) ) / sizeof ( (x) [0] ) )

	// makros um anzahl der befehle und anzahl der codecs zu bestimmen

	#define NUM_COMMANDS 			( NUM_ARRAY_ELEMENTS (astCmdList) 		)
	#define NUM_VALUE_CODECS  ( NUM_ARRAY_ELEMENTS (astValueCodecs) )


	// Prototyp fr intergrierte Help-Funktion
	void vCmdHelp (UCHAR ucCmd, T_CMD_DATA_ELEMENTS *pstElements);


	// Datentyp fr einen Command
	typedef struct
	{
		T_PF_PARSE_CALL	pfCallFunction;
		UCHAR  						aucText [MAX_STRING_LEN];
	} T_COMMAND_DEF;



	// Array aus Value Codec Funktionen zusammenbauen 
	T_PF_VALUE_CODEC astValueCodecs [] = 
	{
		#define PARSE_CODEC(name,func,listname)  (T_PF_VALUE_CODEC) func, 
			#include PARSE_CONFIG_FILE
		#undef  PARSE_CODEC
	};


	// Zwischendatentyp fr Codec Description (String mit fester Lnge)
	typedef UCHAR CODEC_DESCR [MAX_STRING_LEN];

	// Array aus Value Codec Descriptions zusammenbauen
	CODEC_DESCR astValueCodecDesc [] PROGMEM = 
	{
		#define PARSE_CODEC(name,func,listname)  listname, 
			#include PARSE_CONFIG_FILE
		#undef  PARSE_CODEC
	};


	// Array aus Commands
	T_COMMAND_DEF astCmdList [] PROGMEM = {
		#define PARSE_COMMAND(name,string,func)	{ (T_PF_PARSE_CALL) func, string, },
			#ifdef PARSE_USE_HELP
				{ vCmdHelp, 		"help",  },		// Eintrag fr integrierte Hilfe
			#endif
			#include PARSE_CONFIG_FILE
		#undef  PARSE_COMMAND
	};



// setze zeiger weiter bis zeichen != 32

UCHAR ucNextChar (UCHAR **ppcData)
{
	UCHAR *pcData = *ppcData;
	while (*pcData == 32) { pcData ++; }
	*ppcData = pcData;
	return *pcData;
}

// ermittle aktuellen befehl und dekodiere die daten 


BOOL bCheckCommand (UCHAR **ppcData, UCHAR *pucCmd, T_CMD_DATA_ELEMENTS *pstDataElements)
{
  UCHAR ucCmd, 										// zhler ber alle commands (index innerhalb der command-tabelle)
				ucElementPtr = 0;					// index des daten-elementes in pstDataElements
	UCHAR ucPos, 										// zeiger auf zeichen in command-tabelle (Eintrag ucCmd)
				ucChar;										// zeichen aus command-tabelle 
	char 	cState;										// zustand der statemachine (0 = command gefunden, > 0 = suche noch, -1 = fehler)
	UCHAR *pcData   = *ppcData;			// zeiger auf aktuelle stelle im textbuffer
	UCHAR *pcBackup = *ppcData;			// kopie vom zeiger auf aktuelle stelle im textbuffer
	T_CMD_DATA_ELEMENT stElement;		// datenelement um evtl. ergebnis 


	// setze zeiger auf nchste stelle im text die != 32 ist
	do
	{
		if (*pcData == 32) { pcData++; }
	} while (*pcData == 32);
	pcBackup = pcData;

	// initialisiere die bergebenen datenelemente
	for (ucPos = 0; ucPos < MAX_DATA_ELEMENTS; ucPos++)
	{
		(*pstDataElements) [ucPos].ucType = 0;
	}

	// alle eintrge des command-arrays durchklappern
	for (ucCmd = 0; ucCmd < NUM_COMMANDS; ucCmd ++)
	{
		ucPos = 0;				// zeiger auf erstes zeichen des aktuellen commands
		cState = 1;				// erstmal suchen wir noch ...
		ucElementPtr = 0;	// datenelement index resetten
		do
		{
			stElement.ucType = 0; // datenelement typ resetten
			// aktuelles zeichen des aktuellen commands holen
			ucChar = pgm_read_byte (&(astCmdList [ucCmd].aucText [ucPos]));
			
			// ist es ein value codec ?
      if ((ucChar >= 1) && (ucChar <= NUM_VALUE_CODECS))
      {
				cState = -1;
				// value-codec ausfhren
        if (astValueCodecs [ucChar - 1] (&pcData, &stElement))
        {
					// wenn codec erfolgreich ausgefhrt dann weitersuchen
					cState = 1;	
					ucPos++;	
					stElement.ucType = ucChar;
        }
      }
      else
      {
				// was ist es dann ?
  			switch (ucChar)
  			{
  				// ende des commands -> verlassen der statemachine
  				case 0:	
  					cState = 0; 
  					break;
          // space -> werden im text ignoriert (command-zeiger wird erst bei != 32 weitergesetzt)
  				case 32:
  					if (*pcData == 32) { pcData++; }	
  												else { ucPos ++; }
  					break;
          // regulres zeichen -> prfen
  				default :
					  // defaultmssig haben wir einen fehler
  					cState = -1;
  					if (ucChar == *pcData) // aber solange bereinstimmung ->
  					{ 
  						pcData++; 						// weitermachen ....
  						ucPos++; 
  						cState = 1; 
  					}
  					break;
  			}
      }
			// wenn value-codec erfolgreich ausgefhrt (typ liegt vor)
			if ((stElement.ucType != 0) && (ucElementPtr < MAX_DATA_ELEMENTS))
			{
				// speichern und datenelement index eins weiter (bis maximum)
				(*pstDataElements) [ucElementPtr] = stElement;
				ucElementPtr++;
			}
			// statemachine ausfhren solange wie wir noch suchen (1)
		}	while (cState > 0);  
		if (cState == -1) { pcData = pcBackup; }	// bei fehler (command nicht gefunden) alten zeiger wiederherstellen
		if (cState == 0) 	{ break; }							// bei command gefunden -> schleife verlassen
	}
	if (cState == 0) // wenn command gefunden
	{
		// mitteilen ...
	  if (pucCmd != NULL) { *pucCmd = ucCmd; }
		// zeiger weitersetzen (daher brauchen wir zeiger auf zeiger :-))
		*ppcData 	= pcData;
	}
	return cState == 0;	// rckgabe wert : command gefunden oder eben nicht
}




// verkapselung der command-funktion 
// ohne "rckmeldung" des zeigers (geht dann nur fr einen command)

UCHAR ucParserProcess (UCHAR *pucText, BOOL *bFuncCalled)
{
  UCHAR *pucTextCopy = pucText;			// zeiger auf text-buffer (um zeiger auf zeiger zu ermglichen)
	UCHAR  ucCmd = 0xff;							// gefundenes command (wenn denn so)
	T_CMD_DATA_ELEMENTS stElements;		// datenelemente
	T_PF_PARSE_CALL   pfCall;					// fkt die ggf. aufgerufen wird

	// hole command
	if (bCheckCommand (&pucTextCopy, &ucCmd, &stElements))
	{
		// wenn rckmeldung ob ggf. fkt augerufen wurde rcksetzen
		if (bFuncCalled) { *bFuncCalled = FALSE; }
		// funktionszeiger holen
		pfCall = (T_PF_PARSE_CALL) pgm_read_word (&(astCmdList [ucCmd].pfCallFunction));
		if (pfCall != NULL)
		{
			// aufrufen wenn zeiger vorhanden
		  pfCall (ucCmd, &stElements);
			// und ggf rckmeldung das fkt augerufen wurde
			if (bFuncCalled) { *bFuncCalled = TRUE; }
		}
		// gefundenes command zurckgeben
		return ucCmd;
	}
	else
	{
		// kein command gefunden zurckgeben
	  return 0xff;
	}
}



#ifdef PARSE_USE_HELP

	// hilfsfunktion : text aus dem flash ber zeichen-ausgabe-fkt ausgeben
	void vParserPrintTextP (UCHAR *ucProgmem)
	{
		UCHAR ucC;
		do
		{
			ucC = pgm_read_byte (ucProgmem++);
			if (ucC != 0) { vParserWriteChar (ucC); }
		} while (ucC != 0);
	}

	// hilfsfunktion : verkapselung des hilfe-commands (um diesen wie einen
	// 								 regulren befehl behandeln zu knnen)
	void vCmdHelp (UCHAR ucCmd, T_CMD_DATA_ELEMENTS *pstElements)
	{
	 	vParserHelp ();
	}


	// hilfe funktion
	void vParserHelp (void)
	{
	  UCHAR ucC, ucD; // ber alle commands (ucC) und zeichen (ucD) des commands gehen
	  UCHAR ucChar;		// aktuelles zeichen

		// ber alle commands
	  for (ucC = 0; ucC < NUM_COMMANDS; ucC ++) 
	  {
			// ber alle zeichen des commands
	    for (ucD = 0; ucD < strlen_P ((char *) astCmdList [ucC].aucText); ucD ++)
	    {
				// aktuelles zeichen holen
				ucChar = pgm_read_byte (&(astCmdList [ucC].aucText [ucD]));
				// ist es ein value-codec ?
				if ((ucChar >= 1) && (ucChar <= NUM_VALUE_CODECS))
				{
					// value codec descr ausgeben
	      	vParserPrintTextP (astValueCodecDesc [ucChar - 1]);
				}
				else
				{
					// sonst zeichen ausgeben
					vParserWriteChar (ucChar);
				}
	    }
			// und jeweils CRLF ausgeben
	    vParserWriteChar ('\n');
			vParserWriteChar ('\r');
	  }
	}
#endif
