////////////////////////////////////////////////////////////////////////////
//
//  uParse_Codecs.c
//
//	Implementierung der Parser Codecs (Zahlen-, Namen- und String-Codecs)
//
//	Author 	: Rene Bllhoff
//  Created : around Okt. 2007
//
//	(c) 2005-2008 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
//
//////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
////////////////////////////////////////////////////////////////////////////	

	#include "lib.h"


			
		// makros fr bereichsprfungen

		#define ISAREA(x,a,b)   ( ((x) >= (a)) && ((x) <= (b)) )
		#define ISHEX(x)				( ISAREA (x, '0', '9') || ISAREA (x, 'A', 'F') || ISAREA (x, 'a', 'f') )
		#define ISALPHA1st(x)   ( ISAREA (x, 'a', 'z') || ISAREA (x, 'A', 'Z') || ((x) == '_') )
		#define ISALPHA(x)			( ISAREA (x, '0', '9') || ISAREA (x, 'a', 'z') || ISAREA (x, 'A', 'Z') || ((x) == '_') )
		#define HEXUPPER(x)			( ISAREA (x, 'a', 'f') ? (x) ^ 0x20 : (x) )
		#define HEXNIBBLE(x)    ( HEXUPPER(x) >= 'A' ? HEXUPPER(x) - 'A' + 10 : (x) - '0' )


		// makros fr hex-,dez-,bin-werte

		#define ADDHEXNUMBER(x,y)  (x) *= 16; (x) += (HEXNIBBLE (y))
		#define ADDDECNUMBER(x,y)  (x) *= 10; (x) += ((y) - '0')
		#define ADDBINNUMBER(x,y)  (x) *=  2; (x) |= ((y) - '0')



// codec fr hex,dez,bin werte
// 1234  0xffff  $fffe   0b01001010

BOOL bCheckNumber (UCHAR **ppcData, T_CMD_DATA_ELEMENT *pstData)
{
  char cState = 1;
	char cCnt = 0;
	UCHAR *pcData = *ppcData;
	UCHAR cChar;
	ULONG ulTemp = 0;

	do
	{
	  cChar = *pcData;
		
		switch (cState)
		{
			case 1:
				     if (cChar == '0')							{ pcData++; cState = 2; cCnt = 1; }
				else if (ISAREA (cChar, '1', '9'))	{ pcData++; cState = 4; cCnt = 1; ADDDECNUMBER (ulTemp, cChar); }
				else																{ cState = -1; }
				break;
			case 2:
				     if (cChar == 'x')							{ pcData++; cState = 3; }
				else if (cChar == 'b')							{ pcData++; cState = 5; }
				else if (cChar == '0')							{ pcData++; }
				else if (ISAREA (cChar, '1', '9'))	{ cState = 4; cCnt = 1; }
				else																{ ulTemp = 0; cState = 0; }
				break;
			case 3:
				cState = -1; 
				if (cCnt <= 8)
				{
					if (ISHEX (cChar))	          { pcData++; cCnt ++; cState = 3; ADDHEXNUMBER (ulTemp, cChar); }
												else	          { cState = (cCnt > 0) ? 0 : -1; }
				}
				break;
			case 4:
				cState = -1; 
				if (cCnt <= 9)
				{
					if (ISAREA (cChar, '0', '9'))	{ pcData++; cCnt ++; cState = 4; ADDDECNUMBER (ulTemp, cChar); }
																	else	{ cState = (cCnt > 0) ? 0 : -1; }
				}
				break;
			case 5:
				cState = -1; 
				if (cCnt <= 32)
				{
					if (ISAREA (cChar, '0', '1'))	{ pcData++; cCnt ++; cState = 5; ADDBINNUMBER (ulTemp, cChar); }
																	else	{ cState = (cCnt > 0) ? 0 : -1; }
				}
				break;
		}
	} while (cState > 0);
	if (cState == 0) 
	{ 
		*ppcData = pcData; 
		if (pstData != NULL) { pstData->ulLong = ulTemp; }
	}
	return cState == 0;
}



BOOL bCheckName (UCHAR **ppcData, T_CMD_DATA_ELEMENT *pstData)
{
  char cState = 1;
	UCHAR cCnt = 0;
	UCHAR *pcData = *ppcData;
	UCHAR aucBuffer [MAX_STRING_LEN];

	do
	{
		switch (cState)
		{
			case 1:
				if (ISALPHA1st (*pcData))	{ aucBuffer [cCnt] = *pcData; pcData++; cCnt ++; cState = 2; }
													  else	{ cState = -1; }
				break;
			case 2:
				cState = -1;
				if (cCnt < MAX_STRING_LEN)
				{
					if (ISALPHA (*pcData))	{ aucBuffer [cCnt] = *pcData; pcData++; cCnt ++; cState = 2; }
														else	{ cState = (cCnt > 0) ? 0 : -1; aucBuffer [cCnt] = 0; }
				}
		}
	} while (cState > 0);
	if (cState == 0) 
	{ 
		*ppcData = pcData; 
		if (pstData != NULL)
		{
			strcpy ((char *) &(pstData->aucTextBuffer [0]), (char *) aucBuffer);
		}
	}

	return cState == 0;
}


BOOL bCheckString (UCHAR **ppcData, T_CMD_DATA_ELEMENT *pstData)
{
  char cState = 1;
  UCHAR cCnt = 0;  
 	UCHAR *pcData = *ppcData;
	UCHAR aucBuffer [MAX_STRING_LEN];

	do
	{
		switch (cState)
		{
			case 1:
				if (*pcData == '"')	{ pcData++; cState = 2; }
										  else	{ cState = -1; }
				break;
			case 2:
				cState = -1;
				if (cCnt < MAX_STRING_LEN)
				{
               if (*pcData == '"')  { cState = 0; aucBuffer [cCnt] = 0; pcData++;         }
				  else	                    { aucBuffer [cCnt] = *pcData; cCnt++; cState = 2; pcData++;         }
				}
		}
  } while (cState > 0); 
	if (cState == 0)
	{ 
		*ppcData = pcData; 
		if (pstData != NULL)
		{
			strcpy ((char *) &(pstData->aucTextBuffer [0]), (char *) aucBuffer);
		}
	}
	return cState == 0;
}



