////////////////////////////////////////////////////////////////////////////
//
//  uart.c
//
//	Einfache UART-Routinen zur Demonstration des Parsers
//
//	Author 	: Rene Bllhoff
//  Created : around Okt. 2007
//
//	(c) 2005-2008 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
//
//////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
////////////////////////////////////////////////////////////////////////////		

	#include "lib.h"



UCHAR aucRData [INPUT_LENGTH];
UCHAR ucRPtr;
UCHAR ucInputLength;
BOOL  bProcessInput  	= FALSE;

UCHAR ucReceivedChar;
BOOL  bInputReceived 	= FALSE;


void vUARTInitData (void)
{
	memset (aucRData, 0, sizeof (aucRData));

	ucRPtr 				= 0;
	ucInputLength = 0;
	bProcessInput = FALSE;
}


void vUARTInit (void)
{
	vUARTInitData ();

  UBRRL = 17;		// 38400bd @ 11.0592Mhz
	UBRRH =  0;

	UCSRB |= (1 << RXEN)  | (1 << TXEN);
	UCSRC |= (1 << URSEL) | (3 << UCSZ0); 
  UCSRB |= (1 << RXCIE);

	DDRD   |= 0x02;
}


void vUARTSendChar (UCHAR ucValue)
{
	while (!(UCSRA & (1 << UDRE)));
	UDR = ucValue;
}


void vUARTSendString (char *pcData)
{
	while (*pcData)
	{
		vUARTSendChar (*pcData);
		pcData++;
	}
}



void vUARTSendHexUCHAR (UCHAR ucValue)
{
	UCHAR ucX;

	ucX = (ucValue >> 4);
	if (ucX > 9) { ucX += (0x41 - 10); } else { ucX += 0x30; } 
	vUARTSendChar	(ucX);

	ucX = (ucValue & 15);
	if (ucX > 9) { ucX += (0x41 - 10); } else { ucX += 0x30; } 
	vUARTSendChar	(ucX);
}

void vUARTSendHexUINT (UINT uiValue)
{
  vUARTSendHexUCHAR ((uiValue >> 8) & 0xff);
	vUARTSendHexUCHAR ((uiValue >> 0) & 0xff);
}


void vUARTSendHexULONG (ULONG ulValue)
{
  vUARTSendHexUINT ((ulValue >> 16) & 0xffff);
	vUARTSendHexUINT ((ulValue >>  0) & 0xffff);
}



ISR (USART_RXC_vect)
{
	UCHAR ucData = UDR;

	// einfache empfangsroutine (nicht sehr schn ....) die ankommende 
	// zeichen in einem empfangspuffer speichert und bei CR ein flag setzt.

	bInputReceived = TRUE;
	ucReceivedChar = ucData;

	// eingangspuffer begrenzen
	if (ucRPtr > INPUT_LENGTH) { ucRPtr = INPUT_LENGTH; }
	// einfache steuerzeichenbearbeitung
	// cr -> eingabe abschlieen und flag setzen
  if (ucData == 0x0d)
	{ 
	  aucRData [ucRPtr] = 0;
		bProcessInput = TRUE; ucInputLength = ucRPtr; ucRPtr = 0;
	}
	// bs -> vorheriges zeichen lschen
	else if (ucData == 0x08)
	{ 
	  if (ucRPtr > 0)
		{
			ucRPtr--;
		  aucRData [ucRPtr] = 0;
		}
	}
	// sonstiges -> in den buffer
	else
	{
		aucRData [ucRPtr] = ucData; 
		ucRPtr++;
	}
}

